// pbmd.c
//
// The Program Formerly Known as pollbatterymonitor.c
//
// The Phidget Interface Kit with the integrated LCD screen is actually
// two separate devices and so you must set up two handles and two sets of 
// handlers
//
// This program connects to a Phidget with LCD which has an ammeter and a
// voltmeter attached. It reads the both and writes the results to a log file
// It then writes summary info to a "snapshot" file including a state.
//

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "stateManager.h"
#include "dutycycle.h"

#define LOG_PATH "/var/log/pbm.log" 
#define ERR_PATH "/var/log/pbm.err" 

char* getProgName(char* path);
void redirectOutput();
int validateArgs(int argc, char* argv[]);
int clearShapFile(char *snapFileName);
int getBootflag(char *bootFileName, char *bootflag);
int setBootflag(char *bootFileName, char *bootflag);

int main(int argc, char* argv[])
{		   
	char *progName = getProgName(argv[0]);
	char *sleepMode;
	FILE *logFile;
	char *snapFileName;
	char *bootFileName;
	int sleepTime; 
	int sleepHourStart;
	int sleepMinStart;   
	int sleepHourEnd;
	int sleepMinEnd;   
	int ret;
	   
	char *bootflag = (char*) malloc( 2 );
	time_t boottime;
	
	time ( &boottime );				 // log time we booted

	redirectOutput();
	
	openlog( progName, LOG_CONS, LOG_USER );
	
	validateArgs( argc, argv );	   
	syslog ( LOG_NOTICE, "Power States: AMBER=%d GREEN=%d", AMBER, GREEN );
	
	// extract args	
	logFile = fopen( argv[1], "a" );	// Open log file
	snapFileName = argv[2];			 // Arg 2 is snapshot file
	bootFileName = argv[3];			 // Arg 3 is boot flag file
	sleepTime = atoi( argv[4] );		// Arg 4 is sleep interval
	if (argc==7) {
		// no sleepmode, no minutes
		sleepMode=strdup(SLEEPMODEDEF);
		sleepHourStart = atoi( argv[5] );   // Arg 5 is sleep hour start
		sleepMinStart = 0;
		sleepHourEnd = atoi( argv[6] );	 // Arg 6 is sleep hour end
		sleepMinEnd = 0;
	} else if (argc==8) {
		// sleepmode, no minutes
		sleepMode=strdup(argv[5]);
		sleepHourStart = atoi( argv[6] );   // Arg 5 is sleep hour start
		sleepMinStart = 0;
		sleepHourEnd = atoi( argv[7] );	 // Arg 6 is sleep hour end
		sleepMinEnd = 0;
	} else if (argc==9) {
		// no sleepmode, with minutes
		sleepMode=strdup(SLEEPMODEDEF);
		sleepHourStart = atoi( argv[5] );   // Arg 5 is sleep hour start
		sleepMinStart = atoi( argv[6] );
		sleepHourEnd = atoi( argv[7] );	 // Arg 6 is sleep hour end
		sleepMinEnd = atoi (argv [8]);
	} else if (argc==10) {
		// sleepmode, with minutes
		sleepMode=strdup(argv[5]);
		sleepHourStart = atoi( argv[6] );   // Arg 5 is sleep hour start
		sleepMinStart = atoi( argv[7] );
		sleepHourEnd = atoi( argv[8] );	 // Arg 6 is sleep hour end
		sleepMinEnd = atoi (argv [9]);
	}

	if (!strncmp(sleepMode,SLEEPMODEREL,strlen(SLEEPMODEREL))) {
		syslog(LOG_DEBUG,"sleepmode is (%s)",sleepMode);
		// I only need to do stuff here. Others already have absolute
		// hours.
		int dutycycle=atoi(argv[6]);
		int offtime=atoi(argv[7]);
		int ret=getsleeptimes(dutycycle,
					offtime,
					&sleepHourStart,
					&sleepMinStart,
					&sleepHourEnd,
					&sleepMinEnd);
		if (ret) {
			syslog(LOG_ERR,"Error from getsleeptimes doing nothing.");
			closelog();
			return -1;
		}
		syslog(LOG_DEBUG,"REL DC=%d, OT=%d, START=%02d:%02d, END=%02d:%02d",
			dutycycle,offtime,sleepHourStart,sleepMinStart,
			sleepHourEnd,sleepMinEnd);
	} else if (!strncmp(sleepMode,SLEEPMODEABS,strlen(SLEEPMODEABS))) {
		syslog(LOG_DEBUG,"sleepmode is (%s)",sleepMode);
	} else if (!strncmp(sleepMode,SLEEPMODEDEF,strlen(SLEEPMODEDEF))) {
		syslog(LOG_DEBUG,"sleepmode is (%s)",sleepMode);
	} else {
		syslog(LOG_ERR,"Bad sleepmode (%s), doing nothing.",sleepMode);
		closelog();
		return -1;
	}
								  
	if (logFile == 0) 
		syslog ( LOG_ERR, "Could not open log file: %s Continuing without logging", argv[1] );	   
	
	clearShapFile(snapFileName); 
	
	ret=getBootflag ( bootFileName, bootflag );   
	if (ret!=0) {
		syslog ( LOG_ERR, "Could not open bootflag file: %s Continuing without logging", bootFileName );	   
		
	}
	setBootflag ( bootFileName, BOOTFLAG_MAN_RESTART );

	fputs("\nYYYY-M-MDD,HH:MM:SS,mVolts,mAmps,State,Spike\n",logFile);

	ret = test_interfacekit(progName,
		logFile,
		snapFileName,
		bootflag,
		boottime,
		sleepTime,
		sleepHourStart,
		sleepMinStart,
		sleepHourEnd,
		sleepMinEnd);// and off we go
		
	fclose ( logFile );
	ret == 0 ? syslog (LOG_NOTICE, "Program closing" ) : syslog (LOG_ERR, "Program closing with error code: %d", ret );
	closelog ();
	free ( bootflag );
	return ret;
}

// extract program name from file path
char* getProgName(char* path)
{
	char* ret = strrchr( path, '/' );	   //returns null if '/' not found
	
	if( ret == NULL )
		return path;
	else
		return ret + 1;
}

// Close out stdout and point to a file cos if we are running as a demon we need to log this somewhere.
void redirectOutput()
{	
	fclose(stdout);
	fclose(stderr);

	stdout = fopen(LOG_PATH, "a");
	stderr = fopen(ERR_PATH, "a");

	return;
}

int validateArgs(int argc, char* argv[])
{
	if ( argc < 7 || argc > 10 ) {									
		syslog (LOG_ERR, "Error: Program started with incorrect args. Usage: %s logfile snapshotfile bootflagfile polltime sleephourstart [sleepminstart] sleephourend [sleepminend]", argv[0]);
		exit(1);
	}

	if (argc==7) {
		syslog (LOG_NOTICE, "Program started correctly with args: logfile=%s, snapshotfile=%s, bootflagfile=%s, polltime=%s  sleepmode=%s, sleephourstart=%s sleephourend=%s", 
		argv[1], argv[2], argv[3], argv[4], "DEFAULT", argv[5], argv[6]);
	} else if (argc==8) {
		syslog (LOG_NOTICE, "Program started correctly with args: logfile=%s, snapshotfile=%s, bootflagfile=%s, polltime=%s  sleepmode=%s, sleephourstart=%s sleephourend=%s", 
		argv[1], argv[2], argv[3], argv[4], argv[5], argv[6], argv[7]);
	} else if (argc==9) {
		syslog (LOG_NOTICE, "Program started correctly with args: logfile=%s, snapshotfile=%s, bootflagfile=%s, polltime=%s  sleepmode=%s, sleepstart=%s:%s sleepend=%s:%s", 
		argv[1], argv[2], argv[3], argv[4], "DEFAULT", argv[5], argv[6], argv[7], argv[8]);
	} else if (argc==10) {
		syslog (LOG_NOTICE, "Program started correctly with args: logfile=%s, snapshotfile=%s, bootflagfile=%s, polltime=%s  sleepmode=%s, sleepstart=%s:%s sleepend=%s:%s", 
		argv[1], argv[2], argv[3], argv[4], argv[5], argv[6], argv[7], argv[8], argv[9]);
	} 

		
	return 0;
}

int clearShapFile(char *snapFileName)
{
	FILE *filesnap = fopen( snapFileName, "w" );   
	 
	if (filesnap == 0) 
	{
		syslog ( LOG_ERR, "Could not open snapshot file: %s", snapFileName );
		return -1;
	}   
	
	fclose( filesnap ); // we need to overwrite it every time we write to it	
	return 0;
}

// The system reboots after a sleep period to ensure everyting is restarted properly
// As part of this "planned" reboot the boot flag file is written to.
// If this file does not have the correct value, then a user has powered off/on the box
// in order to force a wakeup, so we dont want to go asleep straight away
// If the value is S then we have scheduled a wakeup. Anything else, or no file means manual
// override.
int getBootflag(char *bootFileName, char *bootflag)
{
	FILE *fileboot = fopen(bootFileName, "r");
	
	if ( (fileboot == 0) ) 
	{
		syslog ( LOG_INFO, "Could not open bootflag file: %s - assuming manual override \"%s\"", 
			bootFileName, BOOTFLAG_MAN_OVERRIDE );	   
		sprintf(bootflag, BOOTFLAG_MAN_OVERRIDE);
		snprintf(bootflag,2,BOOTFLAG_NORMAL);
		return -1;
	}	
	else // read from boot state file
	{	
		fgets(bootflag, 2, fileboot);	// bootflag should be 'S'	
		syslog ( LOG_INFO, "Opened bootflag file, boot flag is: %s", bootflag); 
		   
		if ( strncmp (bootflag, BOOTFLAG_NORMAL, 1) == 0 )	
			syslog ( LOG_INFO, "Bootflag is normal \"%s\"", BOOTFLAG_NORMAL ); 
		else
		{
			syslog ( LOG_INFO, "Bootflag not normal, assuming manual restart \"%s\"", BOOTFLAG_MAN_RESTART ); 
			snprintf(bootflag, 2, BOOTFLAG_MAN_RESTART);	   
		}
		
		fclose( fileboot );
		return 0;
	}   
}   

// clear bootflag file - an S in this file means we are controlling the reboot (eg the user
// did not manually turn off and on the box
int setBootflag(char *bootFileName, char *bootflag)
{
	FILE *fileboot = fopen(bootFileName, "w");

	if ( (fileboot == 0) ) 
	{ 
		syslog ( LOG_ERR, "Could not open bootflag file: %s", bootFileName );
		return -1;
	}
	else	
	{  
		syslog ( LOG_INFO, "Writing bootflag \"%s\" to file: %s", bootflag, bootFileName );
		fputs(bootflag, fileboot);
		fclose( fileboot );
		return 0;
	}   
}

